﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Diagnostics;
using System.Runtime;
using System.ComponentModel;

namespace ConsoleAttributes
{
    /// <summary>
    /// Прием программистов на работу 
    /// </summary>
    public class Job
    {
        //fields
        /// <summary>
        /// Число претендентов
        /// </summary>
        int n;

        /// <summary>
        /// массивы, задающие свойства претендентов 
        /// </summary>
        Prog_Properties[] cand;
        string[] strCand;
        Prog_Properties pattern;

        Prog_Properties currentScale;
        Random rnd;

        //Constructors
        public Job()
        {
            n = 10;
            cand = new Prog_Properties[n];
            strCand = new string[n];
            rnd = new Random();
        }
        public Job(int n)
        {
            this.n = n;
            cand = new Prog_Properties[n];
            strCand = new string[n];
            rnd = new Random();

        }
        public Job(Prog_Properties[] pp)
        {
            n = pp.Length;
            cand = pp;
            strCand = new string[n];
            rnd = new Random();

        }
        //Properties
        public Prog_Properties Pattern
        {
            set { pattern = value; }
        }
        
        [Programmer(Prog_Properties.C_sharp | Prog_Properties.VB)]
        [Conditional("DEBUG")] 
        public void Standard()
        {
            
        }

        //Methods
        /// <summary>
        /// Формирование свойств кандидатов
        /// Каждое свойство появляется с вероятностью 0.5
        /// </summary>
        public void FormCands()
        {
            int properties = 5;
            int p = 0, q = 0, currentProps = 0;
            string strQ;
            for (int i = 0; i < n; i++)
            {
                currentProps = 0; strQ = "";
                for (int j = 0; j < properties; j++)
                {
                    p = rnd.Next(2);
                    q = (int)Math.Pow(2, j);
                    if (p == 1)
                    {
                        currentProps += q;
                        strQ += (Prog_Properties)q + ", ";
                    }
                }
                cand[i] = (Prog_Properties)currentProps;
                if (strQ != "")
                    strCand[i] = strQ.Remove(strQ.Length - 2);
                else strCand[i] = "";
            }
        }//FormCands
        [method: Description("Метод, позволяющий получить свойства кандидатов")]
        [return: Description("Массив строк со свойствами кандидатов")]
        public string[] GetStrCands()
        {
            return strCand;
        }
        public Prog_Properties[] GetCands()
        {
            return cand;
        }
        /// <summary>
        /// Список кандидатов, обладающих
        /// свойствами, заданных образцом.
        /// </summary>
        public ArrayList CandsHavePat()
        {
            ArrayList temp = new ArrayList();
            for (int i = 0; i < n; i++)
                if ((cand[i] & pattern) == pattern)
                    temp.Add("cand[" + i + "]");
            return temp;
        }

        /// <summary>
        /// Список кандидатов, не обладающих
        /// всеми свойствами, заданных образцом.
        /// </summary>
        public ArrayList CandsHaveNotAllPat()
        {
            ArrayList temp = new ArrayList();
            for (int i = 0; i < n; i++)
                if ((~cand[i] & pattern) == pattern)
                    temp.Add("cand[" + i + "]");
            return temp;
        }
        /// <summary>
        /// Список кандидатов, обладающих
        /// некоторыми свойствами, заданных образцом.
        /// </summary>
        public ArrayList CandsHaveSomePat()
        {
            ArrayList temp = new ArrayList();
            for (int i = 0; i < n; i++)
            {
                currentScale = cand[i] & pattern;
                if (currentScale > 0 && currentScale < pattern)
                    temp.Add("cand[" + i + "]");
            }
            return temp;

        }
        /// <summary>
        /// Список кандидатов, обладающих
        /// только свойствами, заданных образцом.
        /// </summary>
        public ArrayList CandsHaveOnlyPat()
        {
            ArrayList temp = new ArrayList();
            for (int i = 0; i < n; i++)
                if (((cand[i] & pattern) == pattern) &&
                    ((cand[i] & ~pattern) == 0))
                    temp.Add("cand[" + i + "]");
            return temp;
        }
    }
}
